import numpy as np
from .optical_flow import get_displacements
import cv2

_k_params = dict(
    winSize=(15, 15),
    maxLevel=4,
    criteria=(cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, 10, 0.03),
)

_feature_params = dict(maxCorners=1000000, qualityLevel=0.1, minDistance=7, blockSize=7)

def get_cam_flow(image1, image2, n_rows, n_cols, k_params=_k_params, feature_params=_feature_params):
    """ Calculate an optical flow for each image block defined by grid.

    Parameters
    ----------
    image1 : numpy array
        image1 a grayscale image
    image2 : numpy array
        image2 a grayscale image
    n_rows : int
        number of rows in the grid
    n_cols : int
        number of columns in the grid
    k_params: dictionary, Optional
        cv2.calcOpticalFlowPyrLK parameters. For more information visit
        https://opencv-python-tutroals.readthedocs.io/en/latest/py_tutorials/py_video/py_lucas_kanade/py_lucas_kanade.html
    _feature_params: dictionary, Optional
        cv2.goodFeaturesToTrack parameter. for more information please visit
        https://opencv-python-tutroals.readthedocs.io/en/latest/py_tutorials/py_video/py_lucas_kanade/py_lucas_kanade.html


    Return
    ------
        A numpy array of shape (n_rows, n_cols, 2) where each image block is assigned with
        an 2D vector representing its average displacement.
    """
    # Get image blocks
    origins, displacements = get_displacements(image1, image2, k_params, feature_params)

    h, w = image1.shape
    row_pixel_indexes = np.linspace(0, h, n_rows + 1, dtype=np.int)
    col_pixel_indexes = np.linspace(0, w, n_cols + 1, dtype=np.int)
    block_dispalcements = []
    block_origins = []

    # Going through all the grid cells
    for y0, y1 in zip(row_pixel_indexes, row_pixel_indexes[1::]):
        for x0, x1 in zip(col_pixel_indexes, col_pixel_indexes[1::]):
            cx = x0 + (x1 - x0) // 2
            cy = y0 + (y1 - y0) // 2
            # search for those origins within the current grid cell
            idx = np.where((origins[:, 0] > x0) & (origins[:, 0] <= x1) & (origins[:, 1] > y0) & (origins[:, 1] <= y1))[
                0]
            if len(idx) == 0:  # if no origin was found assign a zero displacement
                block_dispalcement = np.array([cx, cy])
            else:
                block_dispalcement = np.mean((displacements[idx] - origins[idx]) + np.array([cx, cy]), axis=0,
                                             dtype=np.int)
            block_dispalcements.append(block_dispalcement)
            block_origins.append(np.array([cx, cy]))

    block_dispalcements = np.array(block_dispalcements).reshape(n_rows, n_cols, 2).astype(np.int)
    block_origins = np.array(block_origins).reshape(n_rows, n_cols, 2).astype(np.int)

    return block_origins, block_dispalcements



import cv2
import numpy as np

_k_params = dict(
    winSize=(15, 15),
    maxLevel=4,
    criteria=(cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, 10, 0.03),
)

_feature_params = dict(maxCorners=1000000, qualityLevel=0.1, minDistance=7, blockSize=7)


def get_displacements(image1, image2, k_params=_k_params, feature_params=_feature_params):
    """ Calculate the displacement between "good" features 
    given two consecuitive grayscale images.

    Parameters
    ----------
    iamge1: numpy array,
        First image

    iamge2: numpy array,
        First image

    k_params: dictionary, Optional
        cv2.calcOpticalFlowPyrLK parameters. For more information visit
        https://opencv-python-tutroals.readthedocs.io/en/latest/py_tutorials/py_video/py_lucas_kanade/py_lucas_kanade.html

    _feature_params: dictionary, Optional
        cv2.goodFeaturesToTrack parameter. for more information please visit
        https://opencv-python-tutroals.readthedocs.io/en/latest/py_tutorials/py_video/py_lucas_kanade/py_lucas_kanade.html
        
    Returns
    -------
        - a set of 2D coordinates representing the features location in the first.
        - a set of 2D coordinates representing the matching features 
            location in the second image.
    """
    p1 = cv2.goodFeaturesToTrack(image1, mask=None, **feature_params)
    if p1 is None:
        return np.array([[0, 0]]), np.array([[0, 0]])

    p2, st, err = cv2.calcOpticalFlowPyrLK(image1, image2, p1, None, **k_params)
    if p2 is None:
        return np.array([[0, 0]]), np.array([[0, 0]])

    if not np.any(st == 1):
        return np.array([[0, 0]]), np.array([[0, 0]])

    origin = p1[st == 1].reshape(-1, 2)
    dispacement = p2[st == 1].reshape(-1, 2)
    return origin, dispacement



from setuptools import setup

with open("README.md", 'r') as f:
    long_description = f.read()

setup(
    name='cam-motion-field',
    version='1.4',
    description='This package provides quick and easy way to compute camera motion field given two consecutive images.',
    license="MIT",
    keywords=['video', 'optical-flow', 'camera', 'camera-motion', 'motion field'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Tamas Suveges',
    url='https://github.com/stamas02/cam_motion_field',
    download_url='https://github.com/stamas02/cam_motion_field/archive/v_014.tar.gz',
    author_email='stamas01@gmail.com',
    packages=['cam_motion_field'],
    install_requires=['opencv-python', 'numpy'],
    classifiers=[
        'Development Status :: 4 - Beta',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3',  # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)
